
import torch
import torch.nn as nn
import torch.nn.functional as F


class SamplingNet(nn.Module):
    def __init__(self, ks=60):
        super(SamplingNet, self).__init__()
        self.ks_dim = ks
        self.fcks = nn.Linear(self.ks_dim, 256)

        self.fc1 = nn.Linear(256, 256)
        self.fc2 = nn.Linear(256, 256)
        self.fc3 = nn.Linear(256, 256)
        self.fc4 = nn.Linear(256, 256)
        self.fc5 = nn.Linear(256, 768)
        self.re = nn.LeakyReLU(inplace=True)

    def forward(self, x):
        x = self.fcks(x)
        x = self.re(x)

        # ResidualBlock1
        x_r = x
        x = self.fc1(x)
        x = self.re(x)
        x = self.fc2(x)
        x = x_r + x
        x = self.re(x)
        # ResidualBlock2
        x_r = x
        x = self.fc3(x)
        x = self.re(x)
        x = self.fc4(x)
        x = x_r + x
        x = self.re(x)

        x = self.fc5(x)
        x = self.re(x)
        return x



class LightfieldNet_XYZ(nn.Module):
    def __init__(self, uv=60):
        super(LightfieldNet_XYZ, self).__init__()
        self.uvdim = uv
        self.fctp = nn.Linear(60, 256)
        self.fcuv = nn.Linear(uv, 256)
        self.fc1 = nn.Linear(512, 512)
        self.fc2 = nn.Linear(512, 512)
        self.fc3 = nn.Linear(512, 512)
        self.fc4 = nn.Linear(512, 512)
        self.fc5 = nn.Linear(512, 512)
        self.fc6 = nn.Linear(512, 512)
        self.fc7 = nn.Linear(512, 512)
        self.fc8 = nn.Linear(512, 256)
        self.fc9 = nn.Linear(256, 3)
        self.re = nn.LeakyReLU(inplace=True)

    def forward(self, x):
        tp = x[:, self.uvdim:]
        uv = x[:, :self.uvdim]

        tp = self.fctp(tp)
        tp = self.re(tp)

        uv = self.fcuv(uv)
        uv = self.re(uv)

        tpuv = torch.cat([tp, uv], 1)

        x = self.fc1(tpuv)
        x = self.re(x)
        # ResidualBlock1
        x_r = x
        x = self.fc2(x)
        x = self.re(x)
        x = self.fc3(x)
        x = x_r + x
        x = self.re(x)
        # ResidualBlock2
        x_r = x
        x = self.fc4(x)
        x = self.re(x)
        x = self.fc5(x)
        x = x_r + x
        x = self.re(x)
        # ResidualBlock3
        x_r = x
        x = self.fc6(x)
        x = self.re(x)
        x = self.fc7(x)
        x = x_r + x
        x = self.re(x)
        x = self.fc8(x + tpuv)
        x = self.re(x)
        x = self.fc9(x)
        x = self.re(x)
        return x



class LightfieldAlphaNet(nn.Module):
    def __init__(self, uv=60):
        super(LightfieldAlphaNet, self).__init__()
        self.uvdim = uv
        self.fctp = nn.Linear(60, 256)
        self.fcuv = nn.Linear(uv, 256)
        self.fc1 = nn.Linear(512, 512)
        self.fc2 = nn.Linear(512, 512)
        self.fc3 = nn.Linear(512, 512)
        self.fc4 = nn.Linear(512, 512)
        self.fc5 = nn.Linear(512, 512)
        self.fc6 = nn.Linear(512, 512)
        self.fc7 = nn.Linear(512, 512)
        self.fc8 = nn.Linear(512, 256)
        self.fc9 = nn.Linear(256, 4)
        self.re = nn.LeakyReLU(inplace=True)


    def forward(self, x):
        tp = x[:, self.uvdim:]
        uv = x[:, :self.uvdim]

        tp = self.fctp(tp)
        tp = self.re(tp)

        uv = self.fcuv(uv)
        uv = self.re(uv)

        tpuv = torch.cat([tp, uv], 1)

        x = self.fc1(tpuv)
        x = self.re(x)
        # ResidualBlock1
        x_r = x
        x = self.fc2(x)
        x = self.re(x)
        x = self.fc3(x)
        x = x_r + x
        x = self.re(x)
        # ResidualBlock2
        x_r = x
        x = self.fc4(x)
        x = self.re(x)
        x = self.fc5(x)
        x = x_r + x
        x = self.re(x)
        # ResidualBlock3
        x_r = x
        x = self.fc6(x)
        x = self.re(x)
        x = self.fc7(x)
        x = x_r + x
        x = self.re(x)

        x = self.fc8(x + tpuv)
        x = self.re(x)
        x = self.fc9(x)

        return F.sigmoid(x)

